/***********************************************************
 * GXEPS a grads metafile to PostScript converter
 * Copyright (c) 1999 Matthias Munnich
 *
 * I regard gxeps as freeware as long as proper credit is given.
 * As I often read lengthy copyright notices I adjusted
 * one which seemed fitting. Here it goes:
 *
 * Permission has been granted to copy and distribute gxeps in any 
 * context, including a commercial application, provided that this notice 
 * is present in user-accessible supporting documentation.
 * 
 * This does not affect your ownership of the derived work itself, and 
 * the intent
 * is to assure proper credit for the authors of gxeps, not to interfere
 * with your productive use of gxeps. If you have questions, ask. 
 * "Derived works" includes all programs that utilize gxeps.
 * Credit must be given in user-accessible documentation.
 * 
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, provided 
 * that the above copyright notice appear in all copies and that both that 
 * copyright notice and this permission notice appear in supporting 
 * documentation.  This software is provided "as is" without express or 
 * implied warranty.
 * 
 * Matthias Munnich mailto:munnich@atmos.ucla.edu
 * ==================================================
 *
 *   $Log: gxeps.c,v $
 *   Revision 1.1.1.1  2002/06/27 19:44:01  cvsadmin
 *   initial GrADS CVS import - release 1.8sl10
 *
 *   Revision 1.1.1.1  2001/10/18 02:00:54  Administrator
 *   Initial repository: v1.8SL8 plus slight MSDOS mods
 *
 *   Revision 0.85  2001/02/01 21:23:45  munnich
 *   1. Another change to the memory reallocation for large polygons.
 *   Now it should always works.
 *   2. New macro PSFONTS to enable PostScript font support.
 *   3. Input can now come from stdin by using option -i -.
 *
 *   Revision 0.84  2001/01/27 06:57:04  munnich
 *   Fixed coordinate line thicknes bug.
 *
 *   Revision 0.83  2000/10/13 16:36:01  munnich
 *   Another bug fix related user defined colors.
 *
 *   Revision 0.82  2000/10/12 22:33:52  munnich
 *   Fixed a bug related to user defined colors and black&white mode.
 *
 *   Revision 0.81  2000/09/20 22:51:42  munnich
 *   Fixed rotated strings
 *
 *   Revision 0.80  2000/09/15 19:19:24  munnich
 *   Implemented "-g" option for true 8.5'' x 11'' inch output size.
 *
 *   Revision 0.79  2000/09/05 01:33:13  munnich
 *   Back to old scaling for letter paper.
 *
 *   Revision 0.78  2000/08/25 20:54:30  munnich
 *   Back to version 0.76. V0.77 was broken.
 *
 *   Revision 0.76  2000/06/02 23:45:10  munnich
 *   *** empty log message ***
 *
 *   Revision 0.75  2000/06/02 23:23:52  munnich
 *   Fixed Orientation comment.
 *
 *   Revision 0.74  2000/04/03 17:28:54  munnich
 *   Fixed 2 bugs:
 *   (i) gsave was not bracketed with a grestore when
 *   neither options -L nor -n were used.
 *   (ii) Undefined colors are now set to gray (no. 15).
 *
 *   There is also a rudimentary font support in gxeps.c.
 *   This only comes into affect when a new metafile format is
 *   introduced.  Everthing should be backward compatible.
 *
 *   Revision 0.73  1998/11/30 10:13:57  m211033
 *   Fixed a 2 bug: Lineswidth and colors can know
 *   change within a polygon. This bug only occurred with
 *   colored streamline plots.
 *
 *   Changed the line breaking method for polygons.
 *
 *   Revision 0.72  1997/12/08 14:16:15  m211033
 *   Added the -R option which suppresses the rotation
 *   for landscape metafiles.
 *
 *   Revision 0.71  1997/11/05 17:26:41  m211033
 *   Added a compile option NEVER_CTL_D which disables
 *   the -d option for DEFAULT_CTL_D 0, i.e. no ^D is added in
 *   any case.
 *
 *   Revision 0.70  1997/04/15 15:14:37  m211033
 *   Added a BORDER macro that controls the
 *   added white space (in points) around the graph
 *   for the bounding box comment.
 *
 * Revision 0.69  1997/04/09  14:38:39  m211033
 * Black background now as large as the bounding box.
 * BoundingBox corrected for black background.
 *
 *   Revision 0.68  1996/11/22 14:10:45  m211033
 *   BoundingBox takes filled rectangulars into account.
 *
 * Revision 0.67  96/11/19  16:46:19  m211033
 * Two bugs were fixed:
 * (i) with DEFAULT_CTL_D 1 the -d option didn't work.
 * (ii) Color change now leads to a flush of stored lines
 *     (Before sometimes a line was drawn in a invisible color.)
 * 
 * Revision 0.66  96/10/23  12:50:26  m211033
 * Fixed the data type of file position.
 * 
 * Revision 0.65  1996/06/26  15:20:29  m211033
 * Inserted a gsave to match the grestore.
 * This fixes an annotation bug.
 * Resolution is reduce to 500dpi since Version 0.64.
 *
 * Revision 0.64  1996/05/15  10:17:00  m211033
 * The cut of filled line did not work. No the old method
 * is used untill drawline and fillline is rewritten for
 * the use of reversepath.
 * Filline now also looks for colinear lines.
 *
 * Revision 0.63  1996/04/26  19:32:57  m211033
 * Got rid off the error prone reallocation for saved point.
 *
 * Revision 0.62  96/04/26  19:08:25  m211033
 * Fixed another bug with multible plots in one metafile.
 * 
 * Revision 0.61  1996/04/26  17:16:57  m211033
 * Now it tries to find straight lines in polygons.
 *
 * Revision 0.6  96/04/26  16:15:45  m211033
 * Tries to find dashed lines to make the EPS output
 * more compact. Unnecessary return(0) in while(1) eliminated.
 * 
 *
 ***********************************************************/
          /* default PS-level */
#define PS_LEVEL 1
          /* set this to 1 for default a4 paper */
#define DEFAULTA4 0
     /* default add control-D at end-of-file (some printer may need this) */
#define DEFAULT_CTL_D 0
   /* if NEVER_CTL_D is defined ^D will never be added (disables -d Option) */
#define NEVER_CTL_D
#define NODEBUG 
#define BORDER 10

/* PS font support */
#define PSFONTS 

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <math.h>


/* Some scales for letter paper */
/* #define LETSCALE 0.0657 */
#define LETSCALE 0.1314 
#define LETXOFF 30
#define LETYOFF 45
/* GrADS scaling 8.'' x 11'' */
#define GASCALE 0.144 
/* Some scales for a4 paper */
#define A4SCALE 0.1134 
#define A4XOFF 71
#define A4YOFF 109
/* default file extensions */
#define IN_EXT "gm"
#define OUT_EXT "eps"

#define PNMAX 4096
/* maximum string size */
#define STRSIZ 300
#ifdef __GNUC__
#define fpos_t long
#endif

struct point {short x; short y;};
struct point bblow, bbhigh;  /* BoundingBox */
struct options {    /* command line options (flags) */
	short pscale, color, ctl_d, comment, pslevel,
	tstamp, reverse, rotate, verbose, label, font;
	};
FILE *infile, *outfile;
fpos_t bboxpos,pagespos,pagepos,blackpos; /* output file position of boundingbox and page # comments */
void boundbox(struct point* pnt); /* compute bounding box */
void printbbox(short opscale,short landscape,struct options *o);  /* print bounding box to PS-file */
void print_options(char *argv[]); /* print command line options */
void drawline(struct point pnts[],short *pcnt,
	      short lcolor,short *colold,short *lastfill,short flush); /* draw a polygon */
int SaveLine(struct point pnts[],struct point dline[]);
void PrintDashLine(struct point dline[]);
short CheckDashLine(struct point pnts[],struct point dline[]);
void SetDline(struct point pnts[],struct point dline[]);
short CheckExpandDashLine(struct point pnts[],struct point dline[]);
void parse_arg(int argc,char *argv[],struct options *o,char **fin, char **fout); /* parse command line */
void openfiles(char **fin, char **fout, short verbose); /* Open files */
void fillline(struct point pnts[],short pcnt); /* fill a polygon */
void initepsf(short landscape,int argc,char *argv[],char *fout,struct options *o); /* write PS-file header */
void init_newpage(short landscape,char *fout,struct options *o); /* New page header */


/* --------------------------------------------------------------------------------- */
int main (int argc, char *argv[])  {
   short cmd,opts[7],coldef[100],lcolor=0, colold=-1, wold=-1, landscape=0, npnmax=1, lastfill=0;
   struct point *pnts, *pnts2;            /* points of a polygon */
   struct options o;                      /* command line options */
   int nodraw=0;                          /* skip draw commands (for string drawing) */
   int pnmax=PNMAX;                        /* maximum length of a polygon */
   register int  i,j;
   fpos_t infile_pos;
   short pcnt,pcntSave,fcnt,filflg=0;
   float red,blue,green;
   char *fin=NULL,*fout=NULL;              /* file names */
   char string[STRSIZ],str[STRSIZ+50];  /* I hope nobody writes longer lines */

   o.pscale=DEFAULTA4;                                           /* default options */
#ifdef NEVER_CTL_D
   o.ctl_d=0;
#else
   o.ctl_d=DEFAULT_CTL_D;
#endif
#ifdef PSFONTS
   o.font=1;
#else
   o.font=0;
#endif
   o.reverse=o.color=o.verbose=o.comment=o.tstamp=o.label=0;
   o.rotate=1;
   o.pslevel=PS_LEVEL;
   landscape=0;
   bblow.x=1200;bblow.y=1200;
   bbhigh.x=0; bbhigh.y=0;
   pnts=(struct point *) malloc(pnmax*sizeof(struct point)); /* initial maximal polygon size */

  for(i=0;i<16;i++) coldef[i]=1;          /* Initialize defined color flag*/
  for(i=16;i<100;i++) coldef[i]=0;

  parse_arg(argc,argv,&o,&fin,&fout);     /* Parse command line arguments */
  openfiles(&fin,&fout,o.verbose);                  /* open files */

  /* Translate the file */

  fcnt = 1;

  /*  printf("size of short: %d\n",sizeof(short)); */
  while (1) {
    fread (&cmd , sizeof(short), 1, infile);

    if (cmd==-11){				/* Draw to */
      fread (opts, sizeof(short), 2, infile);
      if(nodraw) {
	pnts[pcnt+1].x=opts[0]/2;
	pnts[pcnt+1].y=opts[1]/2;
	boundbox(&pnts[pcnt+1]);
	if (lcolor != colold)  {
	  fprintf(outfile,"c%i ",lcolor);
	  colold=lcolor;
	}
	continue;
      }
      pnts[++pcnt].x=opts[0]/2;
      pnts[pcnt].y=opts[1]/2;
      boundbox(&pnts[pcnt]);
      if (pcnt%(pnmax/4)==(pnmax/4)-1){
        if (!filflg) {   /* flush pnts to keep vector counts small */
	  drawline(pnts,&pcnt,lcolor,&colold,&lastfill,-1);
          pnts[0].x=opts[0]/2;
	  pnts[0].y=opts[1]/2;
        } else if(pcnt==pnmax-1) { /* increase pnmax to hold more points */
	  pnmax =PNMAX* ++npnmax;
	  pnts2=(struct point *) malloc(pnmax*sizeof(struct point)); /* new memory array */
	  memcpy(pnts2,pnts,pnmax*sizeof(struct point));    /* copy saved pnts to new array */
	  free(pnts);
	  pnts=pnts2;
	  if(o.verbose)
	    printf("     Memory allocation for polygons increased to %i points.\n",pnmax);
        }
      }
    }
    else if (cmd==-10){				/* Move to */
      if (pcnt) {drawline(pnts,&pcnt,lcolor,&colold,&lastfill,0);}
      fread ((char *) opts, sizeof(short), 2, infile);
      pnts[pcnt].x=opts[0]/2;
      pnts[pcnt].y=opts[1]/2;
      boundbox(&pnts[pcnt]);
      if(nodraw) continue;
    }
    else if (cmd==-4) {				/* Set line width */
      if (pcnt) {
	pcntSave=pcnt;
	drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);
	pnts[pcnt].x=pnts[pcntSave].x;
	pnts[pcnt].y=pnts[pcntSave].y;
      } else
	drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);
      fread ((char *)opts, sizeof(short), 2, infile);
      i = opts[0];
      if (i>12) i=12;
      else if (i<1) i=1;
      if(--i!=wold){
	fprintf(outfile,"w%i ",i);
	wold=i;
      }
    }
    else if (cmd==-3) {				/* Set color */
      if (pcnt) {
	pcntSave=pcnt;
	drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);
	pnts[pcnt].x=pnts[pcntSave].x;
	pnts[pcnt].y=pnts[pcntSave].y;
      }
      fread ((char *)opts, sizeof(short), 1, infile);
      lcolor = opts[0];
      /********      printf("New lcol#=%i\n",lcolor); */
      if (lcolor<0) lcolor=0;
      if (lcolor>99) lcolor=99;
      if (!(coldef[lcolor])) lcolor=15;
      if (nodraw) fprintf(outfile,"c%i ",lcolor);
    }
    else if (cmd==-7){				/* Start fill */
      fread ((char *)opts, sizeof(short), 1, infile);
      filflg = 1;
      /* if(o.verbose) fprintf(outfile, "%%Fflag\n"); */
    }
    else if (cmd==-8){				/* End fill */
      if (pcnt) { 
	  if (lcolor != colold || lastfill==0)  {
	  fprintf(outfile,"g%i ",lcolor);
	  colold=lcolor;
	  }
        fillline(pnts,pcnt); pcnt=0;
        filflg = 0;
	lastfill=1;
      }
    }
    else if (cmd==-6){				/* Rectangle fill */
      if (pcnt) {drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);}
      fread ((char *)opts, sizeof(short), 4, infile);
      if (lcolor != colold || lastfill==0)  {
	fprintf(outfile,"g%i ",lcolor);
	colold=lcolor;
      }
      for(i=0;i<4;i++) opts[i]/=2;
      pnts[0].x=opts[0];
      pnts[0].y=opts[2];
      pnts[1].x=opts[1];
      pnts[1].y=opts[3];
      boundbox(&pnts[0]);
      boundbox(&pnts[1]);
      fprintf(outfile,"%i %i %i %i B\n",
	      opts[0],opts[2],opts[1]-opts[0],opts[3]-opts[2]);
      lastfill=1;
    }
    else if (cmd==-9) {				/* End of plotting */
      /*  fsetpos(outfile,&pagepos); */
	fprintf (outfile,"%%%%Trailer\n%%%%EOF\n");
	/*  if(!o.reverse) printbbox(o.pscale,landscape); */
	/*  if(o.verbose) printf("BBox without offset: %i %i %i %i \n",
                           bblow.x, bblow.y, bbhigh.x, bbhigh.y); */
      if (o.ctl_d) fprintf (outfile,"%c%c",(char)4,'\n');
      if(o.verbose) printf ("     Number of pages = %i\n",fcnt);
      fsetpos(outfile,&pagespos);
      fprintf(outfile,"%i",fcnt);
      fclose (outfile);
      return(0);
    }
    else if (cmd==-1) {				/* Start of plotting */
      fread ((char *)opts, sizeof(short), 2, infile);
      landscape= (opts[1]<opts[0]);  /* ysize < xsize : landscape */
      if(o.rotate && landscape) o.rotate=1; /* no -R option and */
      else o.rotate=0; 
      
      initepsf(landscape,argc,argv,fout,&o);
      colold=-1;
      init_newpage(landscape,fout,&o);
      pcnt = 0;
    }
    else if (cmd==-2) {				/* New Page */
      if (pcnt) drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);
#ifdef DEBUG
     fprintf(outfile,
     " %i %i moveto %i %i lineto %i %i lineto %i %i lineto closepath stroke newpath\n ",
     bblow.x,bblow.y,bblow.x, bbhigh.y,bbhigh.x,bbhigh.y,bbhigh.x,bblow.y);
#endif
      fprintf (outfile,"showpage\n");
      printbbox(o.pscale,landscape,&o);
      fgetpos(infile,&infile_pos);
      fread (&cmd , sizeof(short), 1, infile); 
      fsetpos(infile,&infile_pos);
      if (cmd != -9) {  /* if next command is not end_of_plotting */
      ++fcnt;
      fprintf (outfile,"\n%%%%Page: %i %i\n",fcnt,fcnt); /* print %%Page: for the last page */
         fprintf(outfile,"%%%%BoundingBox: ");
         fgetpos(outfile, &bboxpos);
         fprintf(outfile,"                  \n");
         bblow.x=1200;bblow.y=1200;
         bbhigh.x=0; bbhigh.y=0;
	 colold=-1;
         init_newpage(landscape,fout,&o);
      }
    }
    else if (cmd==-5){				/* Define new color */
      fread ((char *)opts, sizeof(short), 4, infile);
      i = opts[0];
      if (i>15 && i<100) {
          green = ((float)(opts[2]))/255.0;
          if (green<0.0) green=0.0;
          if (green>1.0) green=1.0;
          red = ((float)opts[1])/255.0;
          if (red<0.0) red=0.0;
          if (red>1.0) red=1.0;
          blue = ((float)opts[3])/255.0;
          if (blue<0.0) blue=0.0;
          if (blue>1.0) blue=1.0;
          fprintf(outfile,"blackwhite \n");
                    /* (b/w: mapped into grey scale via green intensity) */
          fprintf(outfile,"  {/g%i {%.5g g} bdef /c%i {g1} bdef}\n",i,1.0-green,i);
	  fprintf(outfile,"  {/c%i {%.5g %.5g %.5g c} bdef\n",i,red,green,blue);
          fprintf(outfile,"/g%i {c%i} def}\n ifelse\n",i,i);
	  coldef[i]=1;
      }
    }
    else if (cmd==-15) {		/* Begin string  drawing */
      if (pcnt) drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1);
      if(o.font) nodraw=1;  
      else fprintf(outfile,"\n%%String {\n",string);  
    }
    else if (cmd==-16) {		/* End Char. String */
      /*  if (pcnt) drawline(pnts,&pcnt,lcolor,&colold,&lastfill,1); */
      fread ((char *)opts, sizeof(short), 7, infile);
                  /* opts[0]=len,   opts[1]=x, opts[2]=y, opts[3]=height,
                   * opts[4]=width, opts[5]=angle
                   */
      if(opts[0]>=STRSIZ) {
	printf("String too long. Recompile with bigger STRSIZ.\n");
	exit(1);
      }
      fread(string,sizeof(short),(opts[0])/sizeof(short)+2,infile);
      i=j=0;
      do { /* quote parentheses */
	if(string[i]=='(' || string[i]==')' ) {
	  str[j++]='\\';
	  str[j++]=string[i++];
	} else
	  str[j++]=string[i++];
      }  while(string[i]!='\0');
      str[j]='\0';
      
      /* fprintf(outfile," c4 /Helvetica findfont [%d 0 0 %d 0 0] makefont setfont\n",
                 opts[4]*85/100,opts[3]*75/100); 
    fprintf(outfile,"/Times-Roman findfont [%d 0 0 %d 0 0] makefont setfont\n",
                opts[4]*100/100,opts[3]*80/100);  
    fprintf(outfile,"/Times-Bold findfont [%d 0 0 %d 0 0] makefont setfont\n",
                opts[4]*75/100,opts[3]*80/100);  */
    /* fprintf(outfile," /Symbol findfont [%d 0 0 %d 0 0] makefont setfont\n",
                opts[4]*85/100,opts[3]*70/100);  */
      if(! o.font) fprintf(outfile,"%%} String ");
      fprintf(outfile,"(%s) %d %d %d %d %d %d LS\n",
                  str,opts[6],opts[4]*85/100,opts[3]*75/100,opts[5],opts[1]/2,opts[2]/2);
      nodraw=0;
    }
    else if (cmd==-20) {		/* Draw button -- ignore */
      fread ((char *)opts, sizeof(short), 1, infile);
    }
    else {
       printf ("   Fatal error: Invalid command \"%i\" found in metafile\"%s\".\n",cmd,fin);
       printf ("   Is \"%s\" really a GrADS (v1.5 or higher) metafile?\n",fin);
      return(1);
    }
  }
}
/* --------------------------------------------------------------------------------- */

void initepsf(short landscape,int argc,char *argv[],char *fout,struct options *o){
       char *gr[16] =  {                      /* default greys */
                  "1", "0", "0.16", "0.46", "0.7", "0.58", "0.1", "0.34",
                  "0.22", "0.82", "0.4", "0.64", "0.28", "0.52", "0.76", "0.5"
             };
       char * col[16] = {          /* colors */
		"1 1 1",           /* white */
		"0 0 0",           /* black */
		"0.94 0 0",        /* red */
		"0 0.86 0",        /* green */
		"0.12 0.24 0.98",  /* blue */
		"0 0.78 0.78",     /* cyan */
		"0.86 0 0.39",     /* magenta */
		"0.9 0.86 0.2",    /* yellow */
		"0.94 0.51 0.16",  /* orange */
		"0.63 0 0.78",     /* purple */
		"0.63 0.9 0.2",    /* yellow/green */
		"0 0.63 0.9",      /* med. blue */
		"0.9 0.69 0.18",   /* dark yellow */
		"0 0.82 0.55",     /* aqua */
		"0.42 0 0.86",     /* dark purple */
		"0.5 0.5 0.5"      /* grey */
};
      char *widths[12] = {                      /* widths */
	/*  "2 w","7 w","10 w","14 w","17 w","20 w",
                "24 w","27 w","31 w","34 w","38 w","41 w" old widths */
               "1 w","3 w","5 w","7 w","9 w","10 w",
                "12 w","14 w","15 w","17 w","19 w","21 w" 
             };
      int i;
      char buf[130];
      time_t thetime=time(NULL);
      struct tm *ltime=localtime(&thetime);
      if(o->pslevel>1) fprintf(outfile, "%%!PS-Adobe-2.0 EPSF-1.2\n");
      else             fprintf(outfile, "%%!PS-Adobe-1.0 EPSF-1.2\n");
      fprintf(outfile,"%%%%BoundingBox: ");
      fgetpos(outfile, &bboxpos);
      fprintf(outfile,"                    \n");
      fprintf(outfile,
              "%%%%Creator: gxeps $Revision: 1.1.1.1 $\n"
              "%%%%Title: %s\n"
              "%%%%CreationDate: %s"
              "%%%%Pages: ",fout,ctime(&thetime));
      fgetpos(outfile, &pagespos);
      if(o->pscale !=2) {
	fprintf(outfile,"    \n%%%%PageOrder: Ascend\n%%%%PaperSize: ");
	if(o->pscale==1) fprintf(outfile, "A4\n");
	else      fprintf(outfile, "Letter\n");
      }
      if(landscape) fprintf(outfile, "%%%%PageOrientation: Landscape\n");
      else      fprintf(outfile, "%%%%PageOrientation: Portrait\n");
      fprintf(outfile,"%%%%DocumentFonts: ");
      if(o->tstamp||o->label) fprintf(outfile,"Helvetica");
      fprintf(outfile,"\n%%%%EndComments\n");
      fprintf(outfile,"%%GXEPSCommandLine: ");
      for(i=0;i<argc;i++) fprintf(outfile," %s",argv[i]);
      if(o->comment) {			/* get user comments */
         fprintf(outfile,"\n\n%%BeginUserComments\n");
         printf("%s%s","Comments to plot=?",
               " (Period (.) on a single line to finish)\n");
        while(1) {
          fgets(buf,130,stdin);
          if(buf[0]=='.' && buf[1]=='\n') break;
          fprintf(outfile,"%% %s",buf);
        }
         fprintf(outfile,"%%EndUserComments");
      }
      fprintf(outfile,"%s",
                      "\n\n%%BeginProlog\n");
      fprintf(outfile,"%%Change the following (values: true, false) to get color or b/w output\n");
      fprintf(outfile,"/blackwhite {%s} def\n",
	      (o->color) ? "false" : "true");
      fprintf(outfile,"%s",
	              "/bdef {bind def} bind def\n"
	              "/edef {exch def} bind def\n"
                      "/in {72 mul} bdef\n"
                      "/cm {in 2.54 div} bdef\n"
                      "/c {setrgbcolor} bdef\n"
                      "/g {setgray} bdef\n"
                      "/w {setlinewidth} bdef\n"
                      "/L {moveto rlineto stroke newpath} bdef\n"
                      "/D {5 -1 roll 0 setdash L []0 setdash} bdef %% dashed line\n"
                      "/P {% Lay out a polygon\n"
                      "    % usage: x_n y_n x_n-1 y_n-1 ... x_0 y_0 n P\n"
                      "    3 1 roll moveto {rlineto} repeat} bdef\n"
                      "/F {P fill newpath} bdef %Fill a polygon\n"
                      "/S {P stroke newpath} bdef %Draw a polygon\n"
                      "/gxfonts {%Usage: <font#> gxfonts\n"
                      "    [/Helvetica /Times-Roman /Times-Italic /Symbol /Helvetica-Bold /Times-Bold] \n"
                      "     exch get findfont} bdef\n"
                      "/sfont {%Usage: <font#> <width> <height> sfont\n"
                      "     /szy edef /szx edef gxfonts [szx 0 0 szy 0 0 ] makefont setfont} def\n"
                      "/LS % Left alligned string\n" 
                      "{ % Usage: <string> <font#> <width> <height> <angle> <pos_x> <pos_y>\n"
                      "   gsave translate 100 div rotate sfont 0 0 moveto show grestore\n"
                      "} def\n"
                      "/RS % Right alligned string\n" 
                      "{ % Usage: <string> <font#> <width> <height> <anle> <pos_x> <pos_y>\n"
                      "   gsave translate 1000 div rotate sfont 0 0 moveto dup stringwith pop \n"
                      "   0 sub rmoveto show grestore\n"
                      "} def\n");
if(o->pslevel>1){
         fprintf(outfile,"%s",
                      "/B {rectfill newpath} bdef %Fill a box, usage: xlow ylow width height B\n");
      }
      else { 
         fprintf(outfile,"%s",
                      "/B { %Fill a box, usage: xlow ylow width height B\n"
                      "     /height exch def /width exch def moveto width 0 rlineto\n"
                      "     0 height rlineto width neg 0 rlineto closepath fill newpath} bdef \n");
      }
      fprintf(outfile,"\n%%Define line widths\n");
      for (i=0;i<12;i++)
          fprintf(outfile,"/w%i {%s} bdef\n",i,widths[i]);

      fprintf(outfile,"\n%%Define colors:\n");
      if (o->reverse) {                       /* switch black and white colors */
          fprintf(outfile,"/c0 {%s g} bdef %%black and white reversed\n",gr[1]);
          fprintf(outfile,"/c1 {%s g} bdef\n",gr[0]);
      } else {
          for (i=0;i<2;i++) fprintf(outfile,"/c%i {%s g} bdef\n",i,gr[i]);
      }
      for (i=2;i<16;i++) fprintf(outfile,"/c%i {%s c} bdef\n",i,col[i]);

      fprintf(outfile,"\n%%Define grays:\n");
      if (o->reverse) {                       /* switch black and white */
          fprintf(outfile,"/g0 {%s g} bdef %%black and white reversed\n",gr[1]);
          fprintf(outfile,"/g1 {%s g} bdef\n",gr[0]);
      } else {
          for (i=0;i<2;i++) fprintf(outfile,"/g%i {%s g} bdef\n",i,gr[i]);
      }
      for (i=2;i<16;i++) fprintf(outfile,"/g%i {%s g} bdef\n",i,gr[i]);
      fprintf(outfile,"\nblackwhite\n");
      fprintf(outfile,"{ %%Black&white output: re-define line colors black, fill colors gray... \n  ");
      for (i=1;i<16;i++) fprintf(outfile,"/c%i {g1} def%s",i,(((i+1)%5)==0) ? "\n  " : " ");
      fprintf(outfile,"}\n{ %%Color output: undefine fill-colors... \n  ");
      for (i=0;i<16;i++) fprintf(outfile,"/g%i {c%i} def%s",i,i,(((i+1)%5)==0) ? "\n  " : " ");
      fprintf(outfile,"\n} ifelse\n");
      

      fprintf(outfile,"%%%%EndProlog\n\n%%%%BeginSetup\n");
      fprintf(outfile,"1 setlinecap 1 setlinejoin\n%%%%EndSetup\n");
      fgetpos(outfile,&pagepos);
      fprintf(outfile,"%%%%Page: 1 1\n");
      return;
}
/* --------------------------------------------------------------------------------- */

void init_newpage(short landscape,char *fout,struct options *o){
      int i;
      char buf[130];
      time_t thetime=time(NULL);
      struct tm *ltime=localtime(&thetime);
      if (o->reverse) {
        fgetpos(outfile, &blackpos);
        fprintf(outfile,"                              %% black background\n");
      }
      if(o->tstamp||o->label) fprintf(outfile,"gsave\n");
      if(o->pscale==1) { /* A4 paper */
        if (o->rotate) {
	  fprintf(outfile,"21 cm 0 translate 90 rotate\n"); /* landscape mode */
          fprintf(outfile,"%i %i translate %%move origin slightly in\n",A4YOFF, A4XOFF);
	} else {
          fprintf(outfile,"%i %i translate %%move origin slightly in\n",A4XOFF, A4YOFF);
	}
	fprintf(outfile,"%f %f scale  %% Units: 1000 = 2 cm\n",A4SCALE, A4SCALE);
      } else if (o->pscale==0){ /* Letter paper */
        if (o->rotate) {
	  fprintf(outfile,"8.5 in 0 translate 90 rotate %% landscape\n");
          fprintf(outfile,"%i %i translate %%move origin slightly in\n",LETYOFF,LETXOFF);
	} else {
          fprintf(outfile,"%i %i translate %%move origin slightly in\n",LETXOFF,LETYOFF);
	}
	fprintf(outfile,"%f %f scale  %%Units: 1000 = .9125 inch\n",LETSCALE, LETSCALE);
      } else if (o->pscale==2){ /* GrADS 8.5''x 11' scaling */
        if (o->rotate) 
	  fprintf(outfile,"8.5 in 0 translate 90 rotate %% landscape\n");
	fprintf(outfile,"%f %f scale  %%Units: 1000 = .9125 inch\n",GASCALE, GASCALE);
      }
      fprintf(outfile,"c1\n");
      if(o->tstamp||o->label) {
	fprintf(outfile,"matrix currentmatrix %% store CTM\n");
        fprintf(outfile,"grestore\nc1 /Helvetica findfont 8 scalefont setfont\n");
	if(o->pscale) fprintf(outfile,"1 cm 28.5 cm translate 0 0 moveto\n");
	else     fprintf(outfile,"0.3 in 10.7 in translate 0 0 moveto\n");
	if(o->tstamp) {
          strftime(buf,130,"%c",ltime);
	  fprintf(outfile,"(%s,  %s) show %%file and time stamp\n", fout,buf);
	}
        if(o->label) {			/* get user label */
          fprintf(outfile,"\n\n%%BeginUserLabel\n");
          printf("   %s%s","Label to print on the plot=?"," (Period (.) on a single line to finish)\n");
          while(1) {
            fgets(buf,130,stdin);
            if(buf[0]=='.' && buf[1]=='\n') break;
            fprintf(outfile,"0 -7 translate 0 0 moveto (");
	    for (i=0;i<strlen(buf)-1;i++){
	      if(buf[i]=='(' || buf[i]==')'|| buf[i]=='\\') fprintf(outfile,"%c",'\\'); 
              fprintf(outfile,"%c",buf[i]);
            }
	    fprintf(outfile,") show\n");
          }
          fprintf(outfile,"%%EndUserLabel\n");
        }
        fprintf(outfile,"setmatrix %% restore CTM\n");
      }
      return;
}
/* --------------------------------------------------------------------------------- */

void drawline(struct point pnts[],short *pcnt,
	      short lcolor,short *colold,short *lastfill,short flush){
   register int i,im,j,colum;
   static short LastLine=0;
   static short firstcall=1;
   static struct point *dline;
   long dx,dy,dx1,dy1;

   /* flush=1; mm debugging */

   if(firstcall) {
      dline=(struct point *) malloc(4*sizeof(struct point)); 
      firstcall=0;
   }
   if(*pcnt==1) {
      if (LastLine==2){ /* there is a saved dashed line */
	     if ((LastLine=CheckExpandDashLine(pnts,dline))==2) {
	       /*	 fprintf(outfile,"%%New lastline=2\n");  */
	        dline[1].x=pnts[1].x; dline[1].y=pnts[1].y;
	     } else { /*  Dashed Line  not expandable => print dashed line */
	        PrintDashLine(dline);
		LastLine=SaveLine(pnts,dline);
	     }
      } else if (LastLine==1) { /* there is a saved non-dashed line */
	/* fprintf(outfile,"%%lastline=0\n"); */
            if((LastLine=CheckDashLine(pnts,dline))==2) {                 
	        SetDline(pnts,dline);
            } else { /* Not New LastLine */
	        PrintDashLine(dline);
		LastLine=SaveLine(pnts,dline);
	    }
      } else { /* no last line saved */
	/*	fprintf(outfile,"%%lastline=0\n"); */
         if (lcolor != (*colold) || (*lastfill) ) {
             fprintf(outfile,"c%i ",lcolor);
             *lastfill=0;
             *colold=lcolor;
         }
	    LastLine=SaveLine(pnts,dline);
      }
      if(flush){ 
   if ( lcolor != (*colold) || (*lastfill) )  {
     fprintf(outfile,"c%i ",lcolor);
     *lastfill=0;
     *colold=lcolor;
   }
	 PrintDashLine(dline);
	 LastLine=0;
      }
   } else { 
        if(LastLine) PrintDashLine(dline);
	if(*pcnt>1){
           if ( lcolor != (*colold) || (*lastfill) ) {
                fprintf(outfile,"c%i ",lcolor);
                *lastfill=0;
                *colold=lcolor;
           }
	   dx=(long)(pnts[*pcnt].x- pnts[(*pcnt)-1].x);
	   dy=(long)(pnts[*pcnt].y- pnts[(*pcnt)-1].y);
	   j=0;
	   colum=1;
           for(i=(*pcnt)-1;i>0;i--) {
              im=i-1;
	      dx1=(long)(pnts[i].x-pnts[im].x);
	      dy1=(long)(pnts[i].y-pnts[im].y);
	      if ( (dx==0&&dy==0) || dx*dy1==dx1*dy && !((dx==0&&dx1!=0)||(dy==0&&dy1!=0)) ) {
		 dx+=dx1;
		 dy+=dy1;
		 (*pcnt)--;
	      } else {
		             /* if old and new line directions disagree */
		 colum+=fprintf(outfile,"%i %i",dx,dy);
                 /* (j++%10==9) ? fprintf(outfile,"%c",'\n') : fprintf(outfile,"%c",' '); */
                 /* (j++%10==9) ? fprintf(outfile,"%c",'\n') : fprintf(outfile,"%c",' '); */
                 if (colum>=70) {
		     fprintf(outfile,"%c",'\n');
		     colum=1;
		 } else 
		     colum+=fprintf(outfile,"%c",' ');
		 dx=dx1;
		 dy=dy1;
	      }
           }
	   fprintf(outfile,"%i %i",dx,dy);
           /* (j++%10==9) ? fprintf(outfile,"%c",'\n') : fprintf(outfile,"%c",' '); */
                 if (colum>=60) {
		     fprintf(outfile,"%c",'\n');
		     colum=1;
		 } else 
		     colum+=fprintf(outfile,"%c",' ');
           colum+=fprintf(outfile,"%i %i ",pnts[0].x,pnts[0].y);
	   if(flush==-1) fprintf(outfile,"%i P\n",*pcnt);
	   else
	     if(*pcnt==1) fprintf(outfile,"L\n");
	     else fprintf(outfile,"%i S\n",*pcnt);
	}
	LastLine=0;
   } 
   *pcnt=0;
}
/* --------------------------------------------------------------------------------- */
int SaveLine(struct point pnts[],struct point dline[]){
  dline[0].x=pnts[0].x;
  dline[0].y=pnts[0].y;
  dline[1].x=pnts[1].x;
  dline[1].y=pnts[1].y;
  dline[2].x=pnts[0].x;
  dline[2].y=pnts[0].y;
  dline[3].x=pnts[0].x;
  dline[3].y=pnts[0].y;
  return 1;
}
/* --------------------------------------------------------------------------------- */
void PrintDashLine(struct point dline[]){
  short SolidLine,dxsolid,dysolid,dxskip,dyskip;
  dxsolid=dline[2].x-dline[0].x;
  dysolid=dline[2].y-dline[0].y;
  dxskip=dline[3].x-dline[2].x;
  dyskip=dline[3].y-dline[2].y;
  SolidLine= ((dxsolid==0)&&(dysolid==0)&&(dxskip==0)&&(dyskip==0));
  if(!SolidLine) fprintf(outfile,"[%i %i] ",
	      (short) (sqrt((float) dxsolid*dxsolid+(float) dysolid*dysolid)+0.5),
	      (short) (sqrt((float) dxskip*dxskip+(float) dyskip*dyskip)+0.5));
  fprintf(outfile,"%i %i %i %i %c\n",dline[1].x-dline[0].x,dline[1].y-dline[0].y,
	  dline[0].x,dline[0].y,SolidLine ? 'L' : 'D');
  return;
}
/* --------------------------------------------------------------------------------- */
short CheckDashLine(struct point pnts[],struct point dline[]){
  short dx,olddx,dy,olddy;
  dx=pnts[1].x-pnts[0].x;
  if (dx!=dline[1].x-dline[0].x) return 1; /* delta x disagree */
  dy=pnts[1].y-pnts[0].y;
  if (dy!=dline[1].y-dline[0].y) return 1; /* delta y disagree */
  if((int) dx *(pnts[0].y-dline[1].y) !=(int) (pnts[0].x-dline[1].x)*dy)
      return 1;  /* line directions disagree */
  else return 2;
}
/* --------------------------------------------------------------------------------- */
short CheckExpandDashLine(struct point pnts[],struct point dline[]){
  if( (dline[3].x-dline[2].x!=pnts[0].x-dline[1].x) ||
      (dline[3].y-dline[2].y!=pnts[0].y-dline[1].y) ||
      (dline[2].x-dline[0].x!=pnts[1].x-pnts[0].x)  ||
      (dline[2].y-dline[0].y!=pnts[1].y-pnts[0].y)    )return 1;
  else return 2;
}
/* --------------------------------------------------------------------------------- */
void SetDline(struct point pnts[],struct point dline[]){
  dline[2].x=dline[1].x;
  dline[2].y=dline[1].y;
  dline[3].x=pnts[0].x;
  dline[3].y=pnts[0].y;
  dline[1].x=pnts[1].x;
  dline[1].y=pnts[1].y;
  return;
  }
/* --------------------------------------------------------------------------------- */
void fillline(struct point pnts[],short pcnt){
   register int i,im;
   /* check if the polygon is a rectangel: */
   if(pcnt==4 &&  pnts[0].x==pnts[1].x && pnts[2].x==pnts[3].x &&
	     pnts[1].y==pnts[2].y && pnts[0].y==pnts[3].y) {
      fprintf(outfile,"%i %i %i %i B\n",
	      pnts[0].x,pnts[0].y,pnts[2].x-pnts[0].x,pnts[2].y-pnts[0].y);
   } else if(pcnt==4 && pnts[0].y==pnts[1].y && pnts[2].y==pnts[3].y &&
	     pnts[1].x==pnts[2].x && pnts[0].x==pnts[3].x) {
      fprintf(outfile,"%i %i %i %i B\n",
	      pnts[0].x,pnts[0].y,pnts[2].x-pnts[0].x,pnts[2].y-pnts[0].y);
   } else {
     long dx,dy,dx1,dy1;
     int j,colum;
	   dx=(long)(pnts[pcnt].x- pnts[(pcnt)-1].x);
	   dy=(long)(pnts[pcnt].y- pnts[(pcnt)-1].y);
	   j=0;
	   colum=1;
           for(i=(pcnt)-1;i>0;i--){
              im=i-1;
	      dx1=(long)(pnts[i].x-pnts[im].x);
	      dy1=(long)(pnts[i].y-pnts[im].y);
	      if ( (dx1==0&&dy1==0) ||
		   dx*dy1==dx1*dy && !((dx==0&&dx1!=0)||(dy==0&&dy1!=0)) ) {
		 dx+=dx1;
		 dy+=dy1;
		 (pcnt)--;
	      } else {      /* if old and new line directions disagree */
		 colum+=fprintf(outfile,"%i %i",dx,dy);
                 /* (j++%10==9) ? fprintf(outfile,"%c",'\n') : fprintf(outfile,"%c",' '); */
                 if (colum>=70) {
		     fprintf(outfile,"%c",'\n');
		     colum=1;
		 } else 
		     colum+=fprintf(outfile,"%c",' ');
		 dx=dx1;
		 dy=dy1;
	      }
           }
	   if(colum>=60)
	       fprintf(outfile,"%i %i\n%i %i %i F\n",dx,dy,pnts[0].x,pnts[0].y,pcnt);
	   else
               fprintf(outfile,"%i %i %i %i %i F\n",dx,dy,pnts[0].x,pnts[0].y,pcnt);
   }
}
/* --------------------------------------------------------------------------------- */

void boundbox(struct point* pnts){  /* compute bounding box */
     if(pnts->x<bblow.x) bblow.x=pnts->x;
     if(pnts->x>bbhigh.x) bbhigh.x=pnts->x;
     if(pnts->y<bblow.y) bblow.y=pnts->y;
     if(pnts->y>bbhigh.y) bbhigh.y=pnts->y;
     return;
}
/* --------------------------------------------------------------------------------- */
void printbbox(short pscale,short landscape,struct options *o){
     fsetpos(outfile, &bboxpos);
     if(pscale) {
        bblow.x= ( (float) bblow.x) *A4SCALE;
        bbhigh.x= ((float) bbhigh.x)*A4SCALE;
        bblow.y= ((float) bblow.y)*A4SCALE;
        bbhigh.y= ((float) bbhigh.y)*A4SCALE;
        if(o->rotate==0) {
          fprintf(outfile,"%i %i %i %i",
               bblow.x+A4XOFF-BORDER,bblow.y+A4YOFF-BORDER,bbhigh.x+A4XOFF+BORDER,
		  bbhigh.y+A4YOFF+BORDER);
	  if(o->reverse) {
            fsetpos(outfile, &blackpos);
            fprintf(outfile,"0 g %i %i %i %i B 1 g ",
               bblow.x+A4XOFF-BORDER,bblow.y+A4YOFF-BORDER,bbhigh.x-bblow.x+2*BORDER,
		    bbhigh.y-bblow.y+2*BORDER);
	  }
        } else {
           fprintf(outfile,"%i %i %i %i",
              595-bbhigh.y-A4XOFF-BORDER,bblow.x+A4YOFF-BORDER,612-bblow.y-A4XOFF+BORDER,
		   bbhigh.x+A4YOFF+BORDER);
	  if(o->reverse) {
            fsetpos(outfile, &blackpos);
            fprintf(outfile,"0 g %i %i %i %i B 1 g ",
		  595-bbhigh.y-A4XOFF-BORDER,bblow.x+A4YOFF-BORDER,bbhigh.y-bblow.y+2*BORDER,
		    bbhigh.x-bblow.x+2*BORDER); 
	  }
	}
     } else {
        bblow.x= ((float) bblow.x)*LETSCALE;
        bbhigh.x= ((float) bbhigh.x)*LETSCALE;
        bblow.y= ((float) bblow.y)*LETSCALE;
        bbhigh.y= ((float) bbhigh.y)*LETSCALE;
        if(o->rotate==0){
           fprintf(outfile,"%i %i %i %i",
               bblow.x+LETXOFF-BORDER,bblow.y+LETYOFF-BORDER,bbhigh.x+LETXOFF+BORDER,
		   bbhigh.y+LETYOFF+BORDER);
	  if(o->reverse) {
            fsetpos(outfile, &blackpos);
            fprintf(outfile,"0 g %i %i %i %i B 1 g ",
               bblow.x+LETXOFF-BORDER,bblow.y+LETYOFF-BORDER,bbhigh.x-bblow.x+2*BORDER,
		    bbhigh.y-bblow.y+2*BORDER);
	  }
	} else {
           fprintf(outfile,"%i %i %i %i",
              612-bbhigh.y-LETXOFF-BORDER,bblow.x+LETYOFF-BORDER,612-bblow.y-LETXOFF+BORDER,
		   bbhigh.x+LETYOFF+BORDER);
	  if(o->reverse) {
            fsetpos(outfile, &blackpos);
            fprintf(outfile,"0 g %i %i %i %i B 1 g ",
		      612-bbhigh.y-LETXOFF-BORDER,bblow.x+LETYOFF-BORDER,
		    bbhigh.y-bblow.y+2*BORDER,bbhigh.x-bblow.x+2*BORDER); 
	  }
	}
     }
       
     fseek(outfile,0L,2); 
}
/* --------------------------------------------------------------------------------- */


void parse_arg(int argc,char *argv[],struct options *o,char **fin, char **fout){
     register int i,j;
     i = 1;
     if(argc==1) print_options(argv);

     for (i=1;i<argc;i++) {

     if (*(argv[i])=='-') {  /* parse options */
       j = 0;
       while (*(argv[i]+(++j))) {
         if      (*(argv[i]+j)=='a') o->pscale = 1;
         else if (*(argv[i]+j)=='1') o->pslevel = 1;
         else if (*(argv[i]+j)=='2') o->pslevel = 2;
         else if (*(argv[i]+j)=='c') o->color = 1;
#ifdef NEVER_CTL_D
         else if (*(argv[i]+j)=='d') ;
#else
         else if (*(argv[i]+j)=='d') o->ctl_d = !o->ctl_d;
#endif
         else if (*(argv[i]+j)=='f') o->font =0;
         else if (*(argv[i]+j)=='g') o->pscale = 2;
         else if (*(argv[i]+j)=='i') {*fin = argv[++i];break;}
         else if (*(argv[i]+j)=='l') o->pscale = 0;
         else if (*(argv[i]+j)=='L') o->label = 1;
         else if (*(argv[i]+j)=='n') o->comment = 1;
         else if (*(argv[i]+j)=='o') {*fout = argv[++i];break;}
         else if (*(argv[i]+j)=='r') o->reverse = 1;
         else if (*(argv[i]+j)=='R') o->rotate = 0;
         else if (*(argv[i]+j)=='s') o->tstamp = 1;
         else if (*(argv[i]+j)=='v') o->verbose = 1;
         else { 
	   fprintf(stderr,"Unknown option: %s\n\n",argv[i]);
	   print_options(argv);
	   exit(1);
	 }
       }
     }
    else  /* No command line "-" */
      *fin=argv[i];
  }
  return;
}
/* --------------------------------------------------------------------------------- */

void print_options(char *argv[]){
	       printf("   This is gxeps $Revision: 1.1.1.1 $\n");
#ifdef NEVER_CTL_D
       fprintf(stderr,"   %s%s%s","Usage: ",argv[0],
	   " [-acilLnRrsv -i <in_file>[."  IN_EXT
           "] -o <out_file>] [<in_file>[."IN_EXT"]].\n");
#else
       fprintf(stderr,"%s%s%s","Usage: ",argv[0],
	   " [-acdilLnRrsv -i <in_file>[."  IN_EXT
           "] -o <out_file>] [<in_file>[."IN_EXT"]].\n");
#endif
       fprintf(stderr,"Options:\n");
       fprintf(stderr,"     -1   PostScript Level 1 output.\n");
       fprintf(stderr,"     -2   PostScript Level 2 output.\n");
       fprintf(stderr,"     -a   A4 paper.\n");
       fprintf(stderr,"     -c   Color output.\n");
#ifndef NEVER_CTL_D
       if(DEFAULT_CTL_D)  fprintf(stderr,"     -d   Do not add Control-D at end.\n");
       else fprintf(stderr,"     -d   Add Control-D at end.\n");
#endif
#ifdef PSFONTS
       fprintf(stderr,"     -f   Do not use PostScript fonts.\n");
#endif
       fprintf(stderr,"     -g   true 8.5'' x 11'' output (GrADS scaling).\n");
       fprintf(stderr,"     -i   <in_file>[."IN_EXT"], '-' = stdin.\n");
       fprintf(stderr,"     -l   letter paper\n");
       fprintf(stderr,"     -L   Ask for a label to be printed on the plot.\n");
       fprintf(stderr,"     -n   Ask for a note to include in postscript file header.\n");
       fprintf(stderr,"     -o   <out_file> (default: basename(in_file)."OUT_EXT", '-' = stdout).\n");
       fprintf(stderr,"     -R   Do not rotate landscape metafiles.\n");
       fprintf(stderr,"     -r   Black background.\n");
       fprintf(stderr,"     -s   Add a file & time stamp.\n");
       fprintf(stderr,"     -v   Verbose.\n");
     exit(8);
     }
/* --------------------------------------------------------------------------------- */

void openfiles(char **fin, char **fout,short verbose){ /* Open files */
  int i;

  if (*fin==NULL) {
    *fin = (char *) malloc(sizeof(char)*150);
     fgets(*fin,150,stdin);
     printf("   Read infile = %s\n",*fin);
  }
  if(strcmp(*fin,"-")==0) infile=stdin;
  else infile = fopen(*fin ,"rb");
  if (infile == NULL) {
    *fin=strcat(*fin,"."IN_EXT);
    infile = fopen(*fin,"rb");
    if (infile == NULL) {
      (*fin)[strlen(*fin)-3]='\0';
      printf ("Input file %s[."IN_EXT"] not found.\n",*fin);
      exit(1);
    }
  }
  /* setvbuf(infile,NULL,_IOFBF,(size_t) 524288L); */
  if (*fout==NULL) {
    *fout = (char *) malloc(sizeof(char)*150);
    strcpy(*fout,*fin);
    for (i=strlen(*fout)-1;i>=0;i--) {
      if((*fout)[i]=='.') {strcpy((*fout)+i+1,OUT_EXT);
	break;
      }
      if(i==0){strcpy((*fout)+strlen(*fout),"."OUT_EXT);}
    }
  }
  if(strcmp(*fout,"-")==0) outfile=stdout;
  else outfile = fopen(*fout,"w");
  if (outfile==NULL) {
    printf ("Error opening output file %s \n",*fout);
    exit(1);
  }
  if(verbose) {
    printf("\n   Gxeps $Revision: 1.1.1.1 $\n");
    printf("     Input file  = %s\n",*fin);
    if(strcmp(*fout,"-")==0) printf("output to stdout\n");
    else printf("     Output file = %s\n",*fout);
  }
  /* setvbuf(outfile,NULL,_IOFBF,(size_t) 524288L); */
  return;
}
/* --------------------------------------------------------------------------------- */
/* --------------------------------------------------------------------------------- */
